/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef DBUSMESSAGEUTIL_H
#define DBUSMESSAGEUTIL_H

#include <QVariant>
#include <QString>

class QByteArray;

/*
 * @brief DBusMessageUtil namespace contains helper routines for D-Bus message parsing.
 *
 * Functions for parsing D-Bus messages. These functions handle conversions from C D-Bus
 * API's GLib types to Qt types. If you do not have any interaction with processes that are
 * written in C, you probably won't need these.
 */
namespace DBusMessageUtil
{
    /*
     * G_TYPE_FUNDAMENTAL_SHIFT:
     *
     * Shift value used in converting numbers to GObject type IDs.
     */
    #define G_TYPE_FUNDAMENTAL_SHIFT (2)

    /*
     * Metadata types sent over DBus connection as GObject type
     *
     * G_TYPE_CHAR      3
     * G_TYPE_BOOLEAN   5
     * G_TYPE_INT       6
     * G_TYPE_UINT      7
     * G_TYPE_INT64     10
     * G_TYPE_UINT64    11
     * G_TYPE_DOUBLE    15
     * G_TYPE_STRING    16
     *
     * G_TYPE_X << G_TYPE_FUNDAMENTAL_SHIFT
     */
    enum MafwMetadataTypes
    {
        CHAR = 3 << G_TYPE_FUNDAMENTAL_SHIFT,
        BOOLEAN = 5 << G_TYPE_FUNDAMENTAL_SHIFT,
        INT = 6 << G_TYPE_FUNDAMENTAL_SHIFT,
        UINT = 7 << G_TYPE_FUNDAMENTAL_SHIFT,
        INT64 = 10 << G_TYPE_FUNDAMENTAL_SHIFT,
        UINT64 = 11 << G_TYPE_FUNDAMENTAL_SHIFT,
        DOUBLE = 15 << G_TYPE_FUNDAMENTAL_SHIFT,
        STRING = 16 << G_TYPE_FUNDAMENTAL_SHIFT
    };


    /*
     * Extracts value of given type from given byte array.
     * This is a front-end for extractString() and extractNumericValue().
     *
     * Supported types are GLib types - also defined in enum
     * MafwMetadataTypes.
     *
     * @param type type of the value to be extracted
     * @param array the array of bytes to be extracted
     * @param error Will be set to true if error occurred.
     * @return extracted value as QVariant
     *
     * @see extractString
     * @see extractNumericValue
     */
    QVariant extractMetadataValue(
            int type, QByteArray& array, bool * const error);

    /*
     * Extracts a zero-trailing string from given byte array.
     *
     * A QString is constructed from the array by finding the first
     * occurrence of C string end character '\\0'. Data until '\\0'
     * is removed from the array and put to the returned QString.
     *
     * @param array Array where the string is extracted from.
     * @param error Will be set to true if error occurred.
     * @return extracted string as QString object or empty string if
     * extracting fails
     */
    QString extractString(QByteArray& array, bool * const error);

    /*
     * Extracts numeric value of given type from a QByteArray.
     *
     * A number of bytes quivalent to size of T is removed from
     * the beginning of the array and converted to T and then returned.
     *
     * If array doesn't have enough bytes to fill a T, argument
     * error is set to true and 0 is returned.
     *
     * @code
     * bool error = false;
     * int number = extractNumericValue(array, int(0), &error);
     * @endcode
     *
     * @param array Array where bytes are extracted from.
     * @param type Type of value to be extracted. Value of this argument is omitted.
     * @param error Will be set to true if error occurred.
     * @return extracted T value
     */
    template <typename T> T extractNumericValue(
            QByteArray& array, T type, bool * const error);

}

#endif // DBUSMESSAGEUTIL_H
