/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_DBUS_INTERFACE_H
#define MAFW_DBUS_INTERFACE_H

#include <QtGlobal>
#include <QString>
#include <QStringList>
#include <QMetaType>

/*----------------------------------------------------------------------------
  Common Service & Object names
  ----------------------------------------------------------------------------*/

#define MAFW_SERVICE "com.nokia.mafw"
#define MAFW_INTERFACE "com.nokia.mafw"
#define MAFW_OBJECT "/com/nokia/mafw"

/*----------------------------------------------------------------------------
  Common between renderer and source
  ----------------------------------------------------------------------------*/
#define MAFW_EXTENSION_INTERFACE MAFW_INTERFACE ".extension"

/**
 * list_extension_properties:
 *
 * Method for querying available run-time properties of a extension.
 *
 * Returns: a %DBUS_TYPE_ARRAY of %DBUS_TYPE_STRING:s specifying the
 * property names, followed by a %DBUS_TYPE_ARRAY of
 * %DBUS_TYPE_UINT32:s specifying the #GType of the corresponding
 * property.
 */
#define MAFW_EXTENSION_METHOD_LIST_PROPERTIES "list_extension_properties"

/**
 * set_extension_property:
 * @property: the name of the property (%DBUS_TYPE_STRING).
 * @value: the value of the property (%DBUS_TYPE_VARIANT).
 *
 * Sets a run-time property.
 */
#define MAFW_EXTENSION_METHOD_SET_PROPERTY "set_extension_property"

/**
 * get_extension_property:
 * @property: the name of the property (%DBUS_TYPE_STRING).
 *
 * Queries a run-time property of a extension.  Note that this call may
 * block for a longer time.
 *
 * Returns: the name of the property (%DBUS_TYPE_STRING) followed by
 * the value (%DBUS_TYPE_VARIANT), or an error.
 */
#define MAFW_EXTENSION_METHOD_GET_PROPERTY "get_extension_property"

/**
 * property_changed:
 * @property: the name of the property (%DBUS_TYPE_STRING).
 * @value: the value of the property (%DBUS_TYPE_VARIANT).
 *
 * A signal for wrapping MafwExtension::property-changed.
 */
#define MAFW_EXTENSION_SIGNAL_PROPERTY_CHANGED "property_changed"

/**
 * set_name:
 * @name: the new name of the extension (%DBUS_TYPE_STRING).
 *
 * Sets the name (GObject property) of the extension.
 */
#define MAFW_EXTENSION_METHOD_SET_NAME "set_name"

/**
 * get_name:
 *
 * Returns the name (GObject property) of the extension.
 */
#define MAFW_EXTENSION_METHOD_GET_NAME "get_name"

/**
 * error:
 * @domain: the error domain (%DBUS_TYPE_STRING).
 * @code: the error code (%DBUS_TYPE_INT32).
 * @message: the error message (%DBUS_TYPE_STRING).
 *
 * Signals a GError (MafwExtension::error).
 */
#define MAFW_EXTENSION_SIGNAL_ERROR "error"

/*----------------------------------------------------------------------------
  Plugin
  ----------------------------------------------------------------------------*/

#define MAFW_PLUGIN_INTERFACE MAFW_INTERFACE ".plugin"
#define MAFW_PLUGIN_SERVICE MAFW_SERVICE ".plugin"
#define MAFW_PLUGIN_OBJECT MAFW_OBJECT "/plugin"

/**
 * activate:
 *
 * Activates/reactivates plugin.
 */
#define MAFW_PLUGIN_SIGNAL_ACTIVATE "activate"

/**
 * deactivate:
 *
 * Deactivates plugin.
 */
#define MAFW_PLUGIN_SIGNAL_DEACTIVATE "deactivate"

/**
 * The signal sent when exposing new plugin over DBus
 * The signal must have the plugin uuid as argument (%DBUS_TYPE_STRING)
 */
#define MAFW_PLUGIN_SIGNAL_HELLO "new_plugin"

/*----------------------------------------------------------------------------
  Renderer
  ----------------------------------------------------------------------------*/

#define MAFW_RENDERER_INTERFACE MAFW_INTERFACE ".renderer"
#define MAFW_RENDERER_SERVICE MAFW_SERVICE ".renderer"
#define MAFW_RENDERER_OBJECT MAFW_OBJECT "/renderer"

/**
 * play:
 *
 * Starts playback
 */
#define MAFW_RENDERER_METHOD_PLAY "play"

/**
 * play_object:
 * @object_id: (%DBUS_TYPE_STRING) an object ID to play
 *
 * Starts playback of the given object ID
 */
#define MAFW_RENDERER_METHOD_PLAY_OBJECT "play_object"

/**
 * play_uri:
 * @uri: (%DBUS_TYPE_STRING) a URI to play. The uri is percent encoded at least once, it should decoded in the receiving end
 *
 * Starts playback of the given uri
 */
#define MAFW_RENDERER_METHOD_PLAY_URI "play_uri"


/**
 * play_uri:
 * @uri: (%DBUS_TYPE_STRING) a URI to play. The uri is percent encoded at least once, it should decoded in the receiving end
 * @content_type: (%DBUS_TYPE_STRING) the type of the content pointed by the URI
 * Starts playback of the given uri
 */
#define MAFW_RENDERER_METHOD_PLAY_URI_CONTENT_TYPE "play_uri_content_type"

/**
 * stop:
 *
 * Stops playback
 */
#define MAFW_RENDERER_METHOD_STOP "stop"

/**
 * pause:
 *
 * Suspends playback
 */
#define MAFW_RENDERER_METHOD_PAUSE "pause"

/**
 * pauseAt:
 *
 * Start playback suspended at given position
 */
#define MAFW_RENDERER_METHOD_PAUSE_AT "pause_at"

/**
 * resume:
 *
 * Resumes a suspended playback
 */
#define MAFW_RENDERER_METHOD_RESUME "resume"

/**
 * get_status:
 *
 * Requests the renderer's current status information.
 *
 * Returns: Assigned playlist ID(%DBUS_TYPE_UINT32),
 *          Playback index in the playlist (%DBUS_TYPE_UINT32),
 *          Playback state (%DBUS_TYPE_UINT32 -> %MafwPlayState)
 *          Current object ID (%DBUS_TYPE_STRING)
 */
#define MAFW_RENDERER_METHOD_GET_STATUS "get_status"

/**
 * assign_playlist:
 * @playlist_id: the playlist ID to assign (%DBUS_TYPE_UINT32).
 *
 * Assigns the given playlist to the renderer.
 */
#define MAFW_RENDERER_METHOD_ASSIGN_PLAYLIST "assign_playlist"

/**
 * next:
 *
 * Tells the renderer to skip to the next item in its assigned playlist.
 */
#define MAFW_RENDERER_METHOD_NEXT "next"

/**
 * previous:
 *
 * Tells the renderer to skip to the previous item in its assigned playlist.
 */
#define MAFW_RENDERER_METHOD_PREVIOUS "previous"

/**
 * goto_index:
 *
 * Tells the renderer to skip to the given index in its assigned playlist
 */
#define MAFW_RENDERER_METHOD_GOTO_INDEX "goto_index"

/**
 * set_position:
 * @mode:    Seek mode (#MafwRendererSeekMode) (%DBUS_TYPE_INT32)
 * @seconds: The position to seek to in seconds (%DBUS_TYPE_INT32)
 *
 * Tells the renderer to seek to the given position within the current media.
 */
#define MAFW_RENDERER_METHOD_SET_POSITION "set_position"

/**
 * set_position:
 *
 * Requests the renderer's current playback position within the current media.
 *
 * Returns: The current playback position in seconds (%DBUS_TYPE_UINT32)
 */
#define MAFW_RENDERER_METHOD_GET_POSITION "get_position"

/**
 * get_current_media_info:
 *
 * Requests the renderer's currently played medium's metadata.
 */
#define MAFW_RENDERER_METHOD_GET_CURRENT_MEDIA_INFO "get_current_media_info"

#define MAFW_RENDERER_SIGNAL_STATE_CHANGED "state_changed"
#define MAFW_RENDERER_SIGNAL_PLAYLIST_CHANGED "playlist_changed"
#define MAFW_RENDERER_SIGNAL_ITEM_CHANGED "media_changed"

/**
 * Playback completed @see MafwRenderer::playbackCompleted
 */
#define MAFW_RENDERER_SIGNAL_PLAYBACK_COMPLETED "playback_completed"

/**
 * buffering_info:
 * @status: buffering status as a percent (0 - 100).
 *
 * Indicates the buffering status. Status 100 means buffering complete
 */
#define MAFW_RENDERER_SIGNAL_BUFFERING_INFO "buffering_info"

/**
 * scanning_info:
 * @status: scanning status as a percent (0 - 100).
 *
 * Indicates the scanning status. Status 100 means scanning complete
 */
#define MAFW_RENDERER_SIGNAL_SCANNING_INFO "scanning_info"

/**
 * metadata_changed:
 * @name:  name of the changed metadatum (%DBUS_TYPE_STRING).
 * @value: values (%MAFW_DBUS_TYPE_GVALUEARRAY).
 *
 * Wraps MafwRenderer::metadata-changed.
 */
#define MAFW_RENDERER_SIGNAL_METADATA_CHANGED "metadata_changed"

/*----------------------------------------------------------------------------
  Source
  ----------------------------------------------------------------------------*/

#define MAFW_SOURCE_INTERFACE MAFW_INTERFACE ".source"
#define MAFW_SOURCE_SERVICE MAFW_SERVICE ".source"
#define MAFW_SOURCE_OBJECT MAFW_OBJECT "/source"

/**
 * browse:
 * @object_id: the object id to start from (%DBUS_TYPE_STRING).
 * @property: asked property (%DBUS_TYPE_STRING)
 * @additional_properties:  additional properties to get from
 *              each result resource node (array of %DBUS_TYPE_STRING)
 * @skip_count: number of elements to skip from the resultset
 *              (%DBUS_TYPE_UINT32).
 * @item_count: number of items to retrieve (%DBUS_TYPE_UINT32).
 *
 * Starts a browse session.  Results will arrive in browse_result()
 * methods via the source proxy interface. Note, in case of error
 * the domain, code and message arguments are added to the message.
 *
 * Returns: the session id (%DBUS_TYPE_UINT32).
 */
#define MAFW_SOURCE_METHOD_BROWSE "browse"

/**
 * cancel_browse:
 * @browse_id: the identification number of the request to cancel
 *	       (%DBUS_TYPE_UINT32)
 *
 * Cancels a browse request intiated by a "browse" message.
 * No answer is expected to this call.
 */
#define MAFW_SOURCE_METHOD_CANCEL_BROWSE "cancel_browse"

struct MafwBrowseResultDBusStruct
{
    int remaining_count;
    uint index;
    QString object_id;
    QByteArray array;
    QString domainstr; //for errors
    uint error_code;
    QString error_message;
};

Q_DECLARE_METATYPE(MafwBrowseResultDBusStruct)
Q_DECLARE_METATYPE(QList<MafwBrowseResultDBusStruct>)

/**
 * query:
 * @query: The well-formatted SPARQL query to be handled by the source (%DBUS_TYPE_STRING).
 *
 * Starts a query session.  Results will arrive in query_result()
 * methods via the source proxy interface.
 *
 * Returns: the session id (%DBUS_TYPE_UINT32).
 */
#define MAFW_SOURCE_METHOD_QUERY "query"

/**
 * cancel_query:
 * @query_id: the identification number of the request to cancel
 *	       (%DBUS_TYPE_UINT32)
 *
 * Cancels a query request intiated by a "query" message.
 * No answer is expected to this call.
 */
#define MAFW_SOURCE_METHOD_CANCEL_QUERY "cancel_query"

struct MafwQueryResultDBusStruct
{
    uint index;
    QStringList resultRow;
};

Q_DECLARE_METATYPE(MafwQueryResultDBusStruct)
Q_DECLARE_METATYPE(QList<MafwQueryResultDBusStruct>)

/**
 * metadata_changed:
 * @object_id: object id (%DBUS_TYPE_STRING).
 *
 * Metdata changed signal
 */
#define MAFW_SOURCE_SIGNAL_METADATA_CHANGED "metadata_changed"

/**
 * container_changed:
 * @object_id: object id (%DBUS_TYPE_STRING).
 *
 * Container changed signal
 */
#define MAFW_SOURCE_SIGNAL_CONTAINER_CHANGED "container_changed"


/*----------------------------------------------------------------------------
  Source proxy
  ----------------------------------------------------------------------------*/

#define MAFW_PROXY_SOURCE_INTERFACE MAFW_INTERFACE ".proxy_source"
#define MAFW_PROXY_SOURCE_OBJECT MAFW_OBJECT "/proxy_source"

/**
 * browse_result:
 * @browse_id: the session id (%DBUS_TYPE_UINT32).
 * @results Array of results:
 * {
 * @remaining_count: items remaining in the session, -1 if not known
 *                   (%DBUS_TYPE_INT32).
 * @index: index of the element in this relative to the whole
 *         resultset (%DBUS_TYPE_UINT32).
 * @object_id: an object id (%DBUS_TYPE_STRING).
 * @metadata: the queried metadata (array of %DBUS_TYPE_STRING);
 *            (key1, value1, key2, value2, ...).
 * @domain:  In case of error, the error domain (%DBUS_TYPE_STRING)
 * @code: In case of error, the error code   (%DBUS_TYPE_INT32)
 * @message: In case of error, the error message (%DBUS_TYPE_STRING)
 * }
 *
 * A chunk of results in the given browse session.
 */
#define MAFW_PROXY_SOURCE_METHOD_BROWSE_RESULT "browse_result"

/**
 * query_result:
 * @query_id: the session id (%DBUS_TYPE_UINT32).
 * @remaining_count: items remaining in the session, -1 if not known
 *                   (%DBUS_TYPE_INT32).
 * @results Array of results:
 * (
 *      @index: index of the element in this relative to the whole
 *              resultset (%DBUS_TYPE_UINT32).
 *      @resultRow: Result row (array of %DBUS_TYPE_STRING).
 * )
 * @code: In case of error, the error code   (%DBUS_TYPE_INT32)
 * @message: In case of error, the error message (%DBUS_TYPE_STRING)
 *
 * A chunk of results in the given browse session. signature: uia(uas)is
 */
#define MAFW_PROXY_SOURCE_METHOD_QUERY_RESULT "query_result"

/*******************************************************************
 * MAFW Playlist interface
 *******************************************************************/
#define MAFW_PLAYLIST_SERVICE	MAFW_SERVICE	".playlist"
#define MAFW_PLAYLIST_INTERFACE	MAFW_INTERFACE	".playlist"
#define MAFW_PLAYLIST_PATH	MAFW_OBJECT	"/playlist"

/**
 * playlist_created: %DBUS_MESSAGE_TYPE_SIGNAL
 * @param id: the ID of the playlist just created (%DBUS_TYPE_UINT32).
 * @param name: the name of the playlist just created (%DBUS_TYPE_STRING)
 * @param size: The current size of the playlist
 * @param currentIndex: The currently selected index in the playlist
 *
 * Informs about the creation of a playlist.
 */
#define MAFW_PLAYLIST_SIGNAL_PLAYLIST_CREATED	"playlist_created"

/**
 * get_name:
 *
 * Returns the name of the playlist
 */
#define MAFW_PLAYLIST_METHOD_GET_NAME "get_name"

/**
 * get_repeat:
 *
 * Returns the repeat state of the playlist (%DBUS_TYPE_UINT32)
 */
#define MAFW_PLAYLIST_METHOD_GET_REPEAT "get_repeat"

/**
 * get_item:
 * @index:    an index of an item to get from playlist.  Valid value
 *            range is between 0 and (playlist size - 1).
 *
 * Gets the item at the given playlist index.
 */
#define MAFW_PLAYLIST_METHOD_GET_ITEM "get_item"

/**
 * get_item_media_info:
 * @index:    an index of an MafwMediaInfo item to get from playlist.  Valid value
 *            range is between 0 and (playlist size - 1).
 *
 * Gets the item at the given playlist index.
 */
#define MAFW_PLAYLIST_METHOD_GET_ITEM_MEDIA_INFO "get_item_media_info"

/**
 * get_current:
 *
 * Gets the visual index of the currrent item.
 * reply: %DBUS_MESSAGE_TYPE_METHOD_RETURN or %DBUS_MESSAGE_TYPE_ERROR
 * @idx: visual index of the current item
 * @objectid: object id of the current item
 */
#define MAFW_PLAYLIST_METHOD_GET_CURRENT "get_current"

/**
 * get_next:
 * @index:    visual index of the current item
 *
 * Gets the objectid and the visual index of the next playable item.
 *
 * reply: %DBUS_MESSAGE_TYPE_METHOD_RETURN or %DBUS_MESSAGE_TYPE_ERROR
 * @idx: visual index of the next playable item
 * @objectid: object id of the next playable item
 */
#define MAFW_PLAYLIST_METHOD_GET_NEXT "get_next"

/**
 * get_prev:
 * @index:    visual index of the current item
 *
 * Gets the objectid and the visual index of the previous playable item.
 *
 * reply: %DBUS_MESSAGE_TYPE_METHOD_RETURN or %DBUS_MESSAGE_TYPE_ERROR
 * @idx: visual index of the previous playable item
 * @objectid: object id of the previous playable item
 */
#define MAFW_PLAYLIST_METHOD_GET_PREV "get_prev"

/**
 * get_size:
 *
 * Gets the number of items in the playlist.
 * reply: size of the playlist as %DBUS_TYPE_UINT
 */
#define MAFW_PLAYLIST_METHOD_GET_SIZE "get_size"

/**
 * MAFW_PLAYLIST_CONTENTS_CHANGED:
 * A signal telling that the contents of a shared playlist have changed.
 */
#define MAFW_PLAYLIST_CONTENTS_CHANGED "contents_changed"

/**
 * MAFW_PLAYLIST_UPDATED:
 * A signal telling that current and/or following item in playlist has changed
 */
#define MAFW_PLAYLIST_UPDATED "playlist_updated"

/**
 * MAFW_PLAYLIST_REPEAT_MODE_CHANGED:
 * A signal telling that the repeat mode of a shared playlist has changed.
 */
#define MAFW_PLAYLIST_REPEAT_MODE_CHANGED "repeat_mode_changed"

/**
 * MAFW_PLAYLIST_ITEM_MOVED:
 * A signal telling that an item has been moved to a new place
 */
#define MAFW_PLAYLIST_ITEM_MOVED "item_moved"

/**
 * MAFW_PLAYLIST_PROPERTY_CHANGED:
 * A signal telling that one or more properties of a playlist
 * (name, or repeating mode) of a shared playlist have changed.
 */
#define MAFW_PLAYLIST_PROPERTY_CHANGED "property_changed"

/**
 * get_sort_order:
 *
 * Gets the current sorting order of the playlist.
 * reply: sorting order of the playlist as %DBUS_TYPE_UINT
 */
#define MAFW_PLAYLIST_METHOD_GET_SORT_ORDER "get_sort_order"

/*******************************************************************
 * MAFW Playlist modle interface
 * This is intended only for MAFW INTERNAL USE
 *******************************************************************/
#define MAFW_PLAYLIST_MODEL_INTERFACE MAFW_INTERFACE  ".playlistmodel"
#define MAFW_PLAYLIST_MODEL_PATH  MAFW_OBJECT "/playlistmodel"

/**
 * get_data:
 *
 * Get the data stored under the given row and column.
 * param: row the row where requested data lies as %DBUS_TYPE_INT
 * param: column the row where requested data lies %DBUS_TYPE_INT
 * reply: requested data as %DBUS_TYPE_VARIANT
 */
#define MAFW_PLAYLIST_MODEL_GET_DATA    "get_data"

/**
 * get_column_count:
 *
 * Get the count of columns in the playlist.
 * reply: count of columns as %DBUS_TYPE_INT
 */
#define MAFW_PLAYLIST_MODEL_COLUMN_COUNT "get_column_count"


/**
 * get_row_count:
 *
 * Get the count of rows in the playlist.
 * reply: count of rows as %DBUS_TYPE_INT
 */
#define MAFW_PLAYLIST_MODEL_ROW_COUNT "get_row_count"

/**
 * get_header_data:
 *
 * Get the header data for the given column.
 * param: column the column which header is requested %DBUS_TYPE_INT
 * reply: requested data as %DBUS_TYPE_VARIANT
 */
#define MAFW_PLAYLIST_MODEL_GET_HEADER    "get_header_data"

/**
 * A signal telling that playlist model was reseted and needs to be fetched again. (e.g. when shuffled)
 */
#define MAFW_PLAYLIST_MODEL_SIGNAL_RESET "model_reset"

/**
 * A signal telling that rows were added to playlist model.
 * @start First row index (%DBUS_TYPE_UINT32)
 * @end Last row index (%DBUS_TYPE_UINT32)
 */
#define MAFW_PLAYLIST_MODEL_SIGNAL_ROWS_INSERTED "model_rows_inserted"

/**
 * A signal telling that rows were removed from playlist model.
 * @start First row index (%DBUS_TYPE_UINT32)
 * @end Last row index (%DBUS_TYPE_UINT32)
 */
#define MAFW_PLAYLIST_MODEL_SIGNAL_ROWS_REMOVED "model_rows_removed"

/*----------------------------------------------------------------------------
  Register service
  ----------------------------------------------------------------------------*/

/**
  * Registry service interface definition
  */
#define MAFW_REGISTRY_INTERFACE MAFW_INTERFACE ".registry"

/**
  * The registry object path
  */
#define MAFW_REGISTRY_PATH MAFW_OBJECT "/registry"

/**
  * The signal telling that new source is registered
  * @param name of the plugin (%DBUS_TYPE_STRING)
  * @param uuid of the extension (%DBUS_TYPE_STRING)
  * @param name of the extension (%DBUS_TYPE_STRING)
  */
#define MAFW_REGISTRY_NEW_SOURCE "new_source"

/**
  * The signal telling that new renderer is registered
  * @param name of the plugin (%DBUS_TYPE_STRING)
  * @param uuid of the extension (%DBUS_TYPE_STRING)
  * @param name of the extension (%DBUS_TYPE_STRING)
  */
#define MAFW_REGISTRY_NEW_RENDERER "new_renderer"

#endif
/* vi: set noexpandtab ts=8 sw=8 cino=t0,(0: */
