/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWSHARED_H
#define MAFWSHARED_H

#include <QObject>
#include <QMetaType>

class MafwRegistry;
class MafwPlaylist;
class MafwSharedPrivate;

/**
 * @brief Acts as glue between registry and actual sources,
 * renderers, plugins and playlists.
 *
 * MafwShared offers the backbone of managing MAFW plugins and constructs
 * proxy objects for their D-Bus-based inter-process communication.
 *
 * MafwShared is used as process-specific singleton.
 *
 * Usage example:
 * @code
 * MafwShared* sessionBusTracker = new MafwShared();
 * sessionBusTracker->initTracking( MafwRegistry::instance() );
 * sessionBusTracker->activatePlugins();
 * @endcode
 */
class MafwShared : public QObject
{
    Q_OBJECT

public:
    enum TrackFlags
    {
        TrackNothing=0x0,
        TrackPlugins=0x1,
        TrackSources=0x2,
        TrackRenderers=0x4,
        TrackPlaylists=0x8,
        TrackRendererSignals=0x10,
        TrackPlaylistSignals=0x20
    };

private:
    Q_DISABLE_COPY(MafwShared)

    /**
     * Destructor
     */
    virtual ~MafwShared();

protected:
    /**
     * Constructor
     * @param parent The parent
     */
    MafwShared(QObject* parent);

public:

    /**
     * Returns a singleton instance of MafwShared.
     * @note The returned instance will be automatically deleted when program exits.
     * @return MafwShared singleton instance
     */
    static MafwShared* instance();

public:
    /**
      * Initialize sharing.
      *
      * For example to activate known plugin and create proxies for sources:
      * @code
      * MafwShared::instance()->initialize(TrackPlugins|TrackSources);
      * MafwPluginControlInterface* pluginCntrl=MafwRegistry::instance()->pluginControlInterface(KNOWN_PLUGIN_NAME);
      * if( pluginCntrl ) pluginCntrl->activate();
      * @endcode
      * Shared Sources will automatically be added to/removed from MafwRegistry as they come and go.
      * You probably want to activate all out-of-process plugins you use.
      * Otherwise they may disappear at any time.
      * If plugin is not already running it will be automatically started when activate() is called.
      *
      * @param flags What to track and create proxys for.
      * @return True on success.
      *
      * @see MafwPluginControlInterface
      */
    bool initialize(QFlags<MafwShared::TrackFlags> flags);

    /**
     * Request activating all Mafw plugins installed on the system and starts
     * tracking existing and appearing Mafw components.
     *
     * Tracked components are renderers, sources and playlists that are
     * exported in D-Bus session bus. After this function is called, components are
     * added/removed from the provided registry automatically when they show
     * up/get removed from session bus.
     *
     * All activated plugins will be closed after all activator processes have
     * been closed.
     *
     * @deprecated
     * @param registry Instance used to store Mafw components.
     * @return True if tracking is started successfully
     * @see MafwRegistry
     */
    bool initTracking( MafwRegistry* registry );

    /**
     * Starts tracking existence of Mafw components, but does NOT connect to
     * their signals. This method also does NOT activate any Mafw plugins.
     * If there are no active Mafw plugins, no components will appear to the
     * provided registry.
     *
     * trackMafwComponents() does not take control of lifetime of plugins like
     * initTracking() does. It only tracks availability of plugins.
     * Currently active plugins may disappear at any time if no activators
     * are present.
     *
     * @deprecated
     * @param registry Instance used to store Mafw components.
     * @return True if tracking is started successfully
     * @see MafwShared::initTracking()
     * @see MafwShared::connectToPlaylistsSignals()
     * @see MafwRegistry
     */
    bool trackMafwComponents( MafwRegistry* registry );

    /**
     * Starts to listen signals of shared playlists. This method is needed if
     * you are interested in playlist signals, but using
     * MafwShared::trackMafwComponents(). Does nothing if
     * trackMafwComponents() or initTracking has not been called.
     *
     * @deprecated
     * @see MafwShared::trackMafwComponents()
     */
    void connectToPlaylistsSignals();

    /**
     * Publishes playlist over DBus and adds playlist to registry. This
     * method does not work before initTracking is called.
     * @code
     * MafwShared* m_dbus = MafwShared::instance();
     *
     * ...
     *
     * MafwPlaylist* playlist = new MafwInMemoryPlaylist( "mafwPlaylist_1" );
     * bool published = m_dbus->publishPlaylist( playlist );
     * if (!published)
     * {
     *     delete playlist;
     * }
     *
     * ...
     *
     * m_dbus->unPublishPlaylist(playlistId);
     * @endcode
     *
     * @param playlist The playlist to be published. Ownerships transfers to
     * registry.
     * @see MafwRegistry
     * @note Playlists must NOT be added directly to registry.
     * @return True if publishing succeeded, false otherwise.
     * @note If publishing is unsuccessful, ownership is not transferred.
     */
    bool publishPlaylist( MafwPlaylist* playlist );

    /**
     * Removes playlist publishing from DBus, removes playlist from registry
     * and destroys it. This method does not work before initTracking() is
     * called. Only playlists which are published previously can be
     * unpublished.
     * @code
     * MafwPlaylist* playlist = new MafwCFPlaylist( name, model, this );
     * bool published = m_dbus->publishPlaylist( playlist );
     * if (!published)
     * {
     *     delete playlist;
     * }
     *
     * ...
     *
     * m_dbus->unPublishPlaylist(playlistId);
     * @endcode
     * @param id ID of the playlist to be removed.
     * @return True if unpublishing succeeded, false otherwise.
     * @see MafwPlaylist::id()
     */
    bool unPublishPlaylist( const QString& id );

private:
    MafwSharedPrivate* d;
};

#endif /*MAFWSHARED_H*/
